import bpy
from bpy.types import Operator
from .utilities import *
import os
from .constants import anim_speed_list
import random


def import_model(perso, properties, file_path):
    print('### import function')

    if properties.hd:
        res = '_hd'
    if properties.low:
        res = '_lp'

    collection = perso + res
    print('--- collection name :', collection)

    # Vérifie si la collection existe déjà et s'assure qu'elle n'est pas vide
    for c in bpy.context.scene.collection.children:
        if c.name == collection:
            print('--- collection déjà présente')
            if not len(c.objects):
                print('--- collection vide, suppression')
                bpy.data.collections.remove(c)
            break

    full_path_perso = os.path.join(file_path, 'Collection')

    if not bpy.data.objects.get(collection):
        print('--- objet non trouvé, chargement')
        bpy.ops.wm.append(filename=collection, directory=full_path_perso)
    else:
        print('--- objet trouvé début de copie')
        the_collection = bpy.data.collections.get(collection)

        # Créer une nouvelle collection pour la copie
        new_collection = bpy.data.collections.new(the_collection.name + "_Copy")

        # Ajouter la nouvelle collection à la scène
        bpy.context.scene.collection.children.link(new_collection)
        scene_finale = bpy.context.scene

        # Récupérer la scène à laquelle appartient l'objet
        scene_originale = the_collection.objects[0].users_scene[0] if the_collection.objects[0].users_scene else None
        # Changer la scène active pour la scène de l'objet
        bpy.context.window.scene = scene_originale

        # Désélectionner tous les objets
        bpy.ops.object.select_all(action='DESELECT')

        # Parcourir les objets de la collection et les sélectionner
        hide_collection_after = False
        for obj in the_collection.objects:
            try:
                obj.select_set(True)
            except:
                hide_collection_after = True
                include_view_layers(the_collection, hide=False)
                obj.select_set(True)
        bpy.context.view_layer.objects.active = the_collection.objects[0]

        bpy.ops.object.duplicate()

        # Lier les copies à la nouvelle collection
        for obj in bpy.context.selected_objects:
            new_collection.objects.link(obj)
            obj.location = [0, 0, 0]
            try:
                the_collection.objects.unlink(obj)
            except:
                pass

        # Revenir à la scène d'où l'appel est partie
        bpy.context.window.scene = scene_finale

        collection = new_collection.name

    if 'hide_collection_after' in locals() and hide_collection_after:
        include_view_layers(the_collection, hide=True)

    the_collection = bpy.data.collections.get(collection)
    print('### fin de fonction import, return :', the_collection)
    return the_collection

# Loop l'animation
def stash(obj, action, track_name, start_frame):
    tracks = obj.animation_data.nla_tracks
    new_track = tracks.new(prev=None)
    new_track.name = track_name
    strip = new_track.strips.new(action.name, start_frame, action)
    strip.repeat = 500
    strip.frame_start_ui = int(random.random() * -200)
    frame_rate = bpy.context.scene.render.fps
    strip.scale = frame_rate/30 * (1 + (random.random() - 0.5)*0.1)
    new_track.lock = False
    new_track.mute = False
    obj.animation_data.action = None
    return strip.scale


def set_speed(animations, strip_scale, index):
    speed = 1
    # frame_rate = bpy.context.scene.render.fps
    for i, a in enumerate(anim_speed_list.items()):
        if a[0] in animations[index]:
            speed = a[1]/strip_scale[index]
            return speed


def make_plane():
    vertices = []
    for i in range(4):
        vertices.append((0, 0, 0))
    edges = [(0, 1), (1, 2), (2, 3), (3, 0)]
    faces = [(0, 1, 2, 3)]
    mesh_data = bpy.data.meshes.new("plane_data")
    mesh_data.from_pydata(vertices, edges, faces)
    mesh_data.update()
    plane = bpy.data.objects.new("Population_Emitter", mesh_data)
    bpy.context.scene.collection.objects.link(plane)
    return plane


def follow_curve_setup(library_path, the_collections, animations, strip_scale):
    if not bpy.data.node_groups.get('Population_Follow_Curve'):
        file_path_gn = library_path + os.path.normpath('/GN/population_gn.blend/NodeTree')
        bpy.ops.wm.append(filename='Population_Follow_Curve', directory=file_path_gn)

    # Nom de la courbe
    curve_name = "population curve"

    # Créer une nouvelle courbe
    curve_data = bpy.data.curves.new(name=curve_name, type='CURVE')
    curve_data.dimensions = '3D'

    # Créer un objet courbe à partir des données de courbe
    curve_object = bpy.data.objects.new(curve_name, curve_data)

    # Ajouter l'objet courbe à la scène
    scene = bpy.context.scene
    scene.collection.objects.link(curve_object)

    # Définir l'objet courbe comme objet actif
    bpy.context.view_layer.objects.active = curve_object
    curve_object.select_set(True)

    # Ajouter le modifier geometry nodes
    geo_nodes = curve_object.modifiers.new(name='population', type='NODES')
    geo_nodes.node_group = bpy.data.node_groups['Population_Follow_Curve'].copy()
    main_node_tree = geo_nodes.node_group

    # Entrer dans le node tree, ajouter les nodes instancier, les connecter, les paramétrer
    for i, the_collection in enumerate(the_collections):
        instancier_node = main_node_tree.nodes.new(type='GeometryNodeGroup')
        instancier_node.node_tree = bpy.data.node_groups.get('instancier')

        # Connecter la node
        for node in main_node_tree.nodes:
            name = node.name
            if name == 'instance_count':
                instance_count_node = node
            if name == 'JOIN_GEOMETRY':
                join_geometry = node
                link(main_node_tree, instancier_node, 0, join_geometry, 0)
            if name == 'A':
                reroute_A = node
                link(main_node_tree, reroute_A, 0, instancier_node, 'A')
            if name == 'BOOLEAN':
                reroute_BOOLEAN = node
                link(main_node_tree, reroute_BOOLEAN, 0, instancier_node, 'Boolean')
            if name == 'REROUTE_CURVE':
                reroute_curve = node
                link(main_node_tree, reroute_curve, 0, instancier_node, 'Curve')
            if name == 'REROUTE_END':
                reroute_end = node
                link(main_node_tree, reroute_end, 0, instancier_node, 'End')
            if name == 'REROUTE_COUNT':
                reroute_count = node
                link(main_node_tree, reroute_count, 0, instancier_node, 'Count')

        instancier_node.inputs['Collection'].default_value = the_collection
        instancier_node.inputs['B'].default_value = i + 1

        # Set la vitesse de déplacement
        speed = set_speed(animations, strip_scale, i)

        instancier_node.inputs['Speed'].default_value = speed


    instance_count_node.inputs[5].default_value = len(the_collections)
    properties = bpy.context.scene.POPULATION_Props
    if properties.single:
        geo_nodes["Input_4"] = 1
        geo_nodes["Input_2"] = 0.0
        geo_nodes["Input_8"] = 0.0
    bpy.ops.object.mode_set(mode='EDIT')
    bpy.ops.wm.tool_set_by_id(name="builtin.draw")
    bpy.context.scene.tool_settings.curve_paint_settings.depth_mode = 'SURFACE'


def classic_setup(library_path, the_collections, animations, strip_scale, cluster_mode = False):
    if not bpy.data.node_groups.get('Population_Classic'):
        file_path_gn = library_path + os.path.normpath('/GN/population_gn.blend/NodeTree')
        bpy.ops.wm.append(filename='Population_Classic', directory=file_path_gn)

    # Créer un objet
    obj = make_plane()

    # Définir l'objet comme objet actif
    bpy.context.view_layer.objects.active = obj
    obj.select_set(True)

    # Ajouter le modifier geometry nodes
    geo_nodes = obj.modifiers.new(name='population', type='NODES')
    geo_nodes.node_group = bpy.data.node_groups['Population_Classic'].copy()
    main_node_tree = geo_nodes.node_group

    properties = bpy.context.scene.POPULATION_Props
    if properties.pre_pick_object:
        geo_nodes["Input_11"] = properties.pre_pick_object

    if cluster_mode:
        geo_nodes["Input_4"] = True
        geo_nodes["Input_5"] = 0.0
    else:
        geo_nodes["Input_5"] = 1.0
        geo_nodes["Input_4"] = False

    # Recherche la node JOIN
    # Connecter la node
    join_node = None
    for node in main_node_tree.nodes:
        name = node.name
        if name == 'JOIN':
            join_node = node
            break

    # Entrer dans le node tree, ajouter les nodes instancier, les connecter, les paramétrer
    for i, the_collection in enumerate(the_collections):
        collection_node = main_node_tree.nodes.new(type='GeometryNodeCollectionInfo')

        collection_node.inputs[0].default_value = the_collection
        link(main_node_tree, collection_node, 0, join_node, -1)

        # Set la vitesse de déplacement
        # speed = set_speed(animations, strip_scale, i)


def grid_setup(library_path, the_collections, animations, strip_scale):
    if not bpy.data.node_groups.get('Population_Grid'):
        file_path_gn = library_path + os.path.normpath('/GN/population_gn.blend/NodeTree')
        bpy.ops.wm.append(filename='Population_Grid', directory=file_path_gn)

    # Créer un objet
    obj = make_plane()
    obj.location = bpy.context.scene.cursor.location

    # Définir l'objet comme objet actif
    bpy.context.view_layer.objects.active = obj
    obj.select_set(True)

    # Ajouter le modifier geometry nodes
    geo_nodes = obj.modifiers.new(name='population', type='NODES')
    geo_nodes.node_group = bpy.data.node_groups['Population_Grid'].copy()
    main_node_tree = geo_nodes.node_group

    # Recherche la node JOIN
    # Connecter la node
    join_node = None
    for node in main_node_tree.nodes:
        name = node.name
        if name == 'JOIN':
            join_node = node
            break

    # Entrer dans le node tree, ajouter les nodes instancier, les connecter, les paramétrer
    for i, the_collection in enumerate(the_collections):
        collection_node = main_node_tree.nodes.new(type='GeometryNodeCollectionInfo')

        collection_node.inputs[0].default_value = the_collection
        link(main_node_tree, collection_node, 0, join_node, -1)

        # Set la vitesse de déplacement
        # speed = set_speed(animations, strip_scale, i)


def stadium_setup(library_path, the_collections, animations, strip_scale):
    print('--- grid setup')
    if not bpy.data.node_groups.get('Population_Stadium'):
        file_path_gn = library_path + os.path.normpath('/GN/population_gn.blend/NodeTree')
        print('--- adresse gn:', file_path_gn)
        bpy.ops.wm.append(filename='Population_Stadium', directory=file_path_gn)

    # Créer un objet
    obj = make_plane()

    # Définir l'objet comme objet actif
    bpy.context.view_layer.objects.active = obj
    obj.select_set(True)

    # Ajouter le modifier geometry nodes
    geo_nodes = obj.modifiers.new(name='population', type='NODES')
    geo_nodes.node_group = bpy.data.node_groups['Population_Stadium'].copy()
    main_node_tree = geo_nodes.node_group

    properties = bpy.context.scene.POPULATION_Props
    if properties.pre_pick_collection:
        geo_nodes["Input_3"] = properties.pre_pick_collection

    # Recherche la node JOIN
    # Connecter la node
    join_node = None
    for node in main_node_tree.nodes:
        name = node.name
        if name == 'JOIN':
            join_node = node
            break

    # Entrer dans le node tree, ajouter les nodes instancier, les connecter, les paramétrer
    for i, the_collection in enumerate(the_collections):
        collection_node = main_node_tree.nodes.new(type='GeometryNodeCollectionInfo')

        collection_node.inputs[0].default_value = the_collection
        link(main_node_tree, collection_node, 0, join_node, -1)

        # Set la vitesse de déplacement
        # speed = set_speed(animations, strip_scale, i)


class POPULATION_OT_import(Operator):
    bl_idname = "population.import"
    bl_label = "Import"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        can_import = False
        if bpy.context.scene.POPULATION_Props.crowd and len(bpy.context.scene.POPULATION_Props.hidden_crowd_list) :
            can_import = True
        if bpy.context.scene.POPULATION_Props.single:
            can_import = True
        return can_import

    def execute(self, context):
        print('')
        print('--- operator import')
        try:
            bpy.ops.object.mode_set(mode='OBJECT')
        except:pass
        library_path = get_addon_preferences().library_path
        the_collections = []
        file_paths = []
        animations = []

        layer_collection = bpy.context.view_layer.layer_collection
        bpy.context.view_layer.active_layer_collection = layer_collection

        ### import du perso
        properties = bpy.context.scene.POPULATION_Props
        crowd_list = properties.hidden_crowd_list

        if properties.single:
            perso = bpy.context.window_manager.people_previews
            perso = perso.split('.jpg')[0].split('#')[0]
            file_name = perso + '.blend'
            file_paths.append(os.path.join(library_path, file_name))
            the_collections.append(import_model(perso, properties, file_paths[0]))
        if properties.crowd:
            properties.hd = False
            properties.low = True
            list_length = len(crowd_list)
            for i in range(list_length):
                person = crowd_list[i]
                perso = person.perso
                file_name = perso + '.blend'
                file_paths.append(os.path.join(library_path, file_name))
                the_collections.append(import_model(perso, properties, file_paths[-1]))

        # randomisation des couleurs
        print('--- RANDOMIZE COLOR')
        for collection in the_collections:
            for obj in collection.objects:
                if obj.type == 'MESH':
                    material = obj.data.materials[0]
                    mat_copy = material.copy()
                    obj.material_slots[0].material = mat_copy
                    # Recherche la node
                    for n in mat_copy.node_tree.nodes:
                        if n.type == 'GROUP':
                            node_name = n.name
                            if properties.randomize_color:
                                print('RANDOM YES')
                                bpy.data.materials[mat_copy.name].node_tree.nodes[node_name].inputs[10].default_value = 1.0
                            else:
                                print('RANDOM NO')
                                bpy.data.materials[mat_copy.name].node_tree.nodes[node_name].inputs[10].default_value = 0.0

        ### import de l'anim
        if properties.single:
            perso = bpy.context.window_manager.people_previews
            perso = perso.split('.jpg')[0].split('#')[0]
            anim = bpy.context.window_manager.anim_previews
            anim = perso + '_' + anim.split('#')[0]
            animations.append(anim)
            full_path_anim = os.path.join(file_paths[0], 'Action')
            print('--- anim :', anim)
            if not bpy.data.actions.get(anim):
                print('--- anim absente, chargement')
                bpy.ops.wm.append(filename=anim, directory=full_path_anim)

        if properties.crowd:
            list_length = len(crowd_list)
            for i in range(list_length):
                person = crowd_list[i]
                perso = person.perso
                anim = perso+'_'+person.anim
                animations.append(anim)
                file_name = perso + '.blend'
                file_path = os.path.join(library_path, file_name)
                full_path_anim = os.path.join(file_path, 'Action')
                bpy.ops.wm.append(filename=anim, directory=full_path_anim)

        ### assigne l'animation à l'armature
        armature = None
        strip_scale = []

        print('--- recherche armature')
        for i, the_collection in enumerate(the_collections):
            for obj in the_collection.objects:
                if obj.type == 'ARMATURE':
                    armature = obj
                    break
            if not armature:
                return {'FINISHED'}

            action = bpy.data.actions.get(animations[i])

            armature.animation_data.action = action

            strip_scale.append(stash(armature, armature.animation_data.action, action.name, 1))

        # Si ambiant et single on place sur le curseur et on quitte
        if properties.ambient and properties.single:
            for obj in the_collection.objects:
                if obj.type == 'ARMATURE':
                    obj.location = bpy.context.scene.cursor.location
            return {'FINISHED'}

        # Cache les collections
        for the_collection in the_collections:
            include_view_layers(the_collection, hide=True)

        if bpy.context.window_manager.crowd_type_previews == 'follow_a_curve' or (properties.single and properties.walk):
            follow_curve_setup(library_path, the_collections, animations, strip_scale)

        elif bpy.context.window_manager.crowd_type_previews == 'classic':
            classic_setup(library_path, the_collections, animations, strip_scale)

        elif bpy.context.window_manager.crowd_type_previews == 'cluster_mode':
            classic_setup(library_path, the_collections, animations, strip_scale, cluster_mode= True)

        elif bpy.context.window_manager.crowd_type_previews == 'grid_mode':
            grid_setup(library_path, the_collections, animations, strip_scale)

        elif bpy.context.window_manager.crowd_type_previews == 'stadium':
            stadium_setup(library_path, the_collections, animations, strip_scale)

        return{'FINISHED'}


class POPULATION_UL_crowd_list(bpy.types.UIList):
    # bl_idname = "population.crowd_list"

    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        # We could write some code to decide which icon to use here...
        custom_icon = 'OBJECT_DATAMODE'

        # Make sure your code supports all 3 layout types
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.label(text='', icon=custom_icon)
            layout.label(text=item.perso)
            layout.label(text=item.anim)

        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon=custom_icon)


class POPULATION_OT_NewItem(Operator):
    """Add a new item to the list."""
    bl_idname = 'population.newitem'

    bl_idname = "population.new_item"
    bl_label = "Add a new person"

    def execute(self, context):
        properties = bpy.context.scene.POPULATION_Props
        properties.crowd_list.add()

        perso = bpy.context.window_manager.people_previews.split('#')[0]
        properties.crowd_list[-1].perso = perso

        anim = bpy.context.window_manager.anim_previews.split('#')[0]
        properties.crowd_list[-1].anim = anim

        for i in range(properties.sub_instance):
            properties.hidden_crowd_list.add()
            properties.hidden_crowd_list[-1].perso = perso
            properties.hidden_crowd_list[-1].anim = anim

        for i in properties.hidden_crowd_list:
            print('---', i.perso)

        return{'FINISHED'}


class POPULATION_OT_RemoveItem(Operator):
    """Delete the selected item from the list."""

    bl_idname = "population.removeitem"
    bl_label = "Remove a person"

    @classmethod
    def poll(cls, context):
        return context.scene.crowd_list_index or (not context.scene.crowd_list_index and len(context.scene.POPULATION_Props.crowd_list))

    def execute(self, context):
        properties = bpy.context.scene.POPULATION_Props
        my_list = properties.crowd_list
        index = context.scene.crowd_list_index

        my_list.remove(index)
        # index = min(max(0, index - 1), len(my_list) - 1)

        for i in range(properties.sub_instance):
            properties.hidden_crowd_list.remove(index*properties.sub_instance)


        return{'FINISHED'}


# class POPULATION_OT_CleanList(Operator):
#     """Clean list"""
#
#     bl_idname = "population.cleanlist"
#     bl_label = "Clean list"
#
#     @classmethod
#     def poll(cls, context):
#         return context.scene.crowd_list_index or (not context.scene.crowd_list_index and len(context.scene.POPULATION_Props.crowd_list))
#
#     def execute(self, context):
#         properties = bpy.context.scene.POPULATION_Props
#         my_list = properties.crowd_list
#
#         for i in my_list:
#             bpy.ops.population.removeitem()
#
#         return{'FINISHED'}