from dataclasses import dataclass
import bpy
from bpy.types import Panel, UILayout, NodesModifier
from .utilities import init_previews

from .operators import POPULATION_OT_import, POPULATION_OT_NewItem, POPULATION_OT_RemoveItem

@dataclass
class Pointer_Prop:
    type_:str
    search_id:str
    icon:str


def get_gn_mod(obj: bpy.types.Object):
    mod = None
    if obj:
        for m in obj.modifiers:
            if m.type == 'NODES' and 'Population' in m.node_group.name:
                mod = m
                break
        return mod
    return None


def draw_gn_mod_props(layout: UILayout, mod: NodesModifier, input):
    data, prop, use_attr, prop_attr = mod, f'["{input.identifier}"]', f'["{input.identifier}_use_attribute"]', f'["{input.identifier}_attribute_name"]'
    pointer_data: dict = {
        'OBJECT': Pointer_Prop('OBJECT', 'objects', 'OBJECT_DATA'),
        'COLLECTION': Pointer_Prop('COLLECTION', 'collections', 'OUTLINER_COLLECTION')
    }
    row = layout.split(align=True, factor=0.4)
    row.alignment = 'RIGHT'

    row.label(text=input.name)

    subrow = row.row(align=True)
    subrow.use_property_split = True
    subrow.use_property_decorate = True

    if input.item_type in pointer_data.keys():
        subrow.prop_search(mod, prop, bpy.data, pointer_data[input.item_type].search_id, text='', icon=pointer_data[input.item_type].icon)
    else:
        final_prop = prop_attr if getattr(mod, use_attr, 0) != 0 else prop
        subrow.prop(data, final_prop, text='', )


class POPULATION_PT_Panel(Panel):
    "Population"
    bl_idname='POPULATION_PT_Panel'
    bl_label = "Population"
    bl_name = "Population"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = 'Bproduction'

    def draw_header(self, context):
        global pop_custom_icons
        self.layout.label(text="", icon_value=pop_custom_icons["population"].icon_id)

    def draw(self, context):
        return


class POPULATION_PT_Import(Panel):
    "Crowd creation"
    bl_idname='POPULATION_PT_Import'
    bl_label = "Crowd creation"
    bl_name = "Crowd creation"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = 'Bproduction'
    bl_parent_id = "POPULATION_PT_Panel"

    def __init__(self):
        init_previews()

    def draw(self, context):
        scale_y = 1.5
        layout = self.layout
        scene = bpy.context.scene
        properties = bpy.context.scene.POPULATION_Props

        # single ou crowd
        line = layout.row(align=True)
        line.scale_y = scale_y * 1.5
        line.prop(properties, 'single', toggle=True, icon='TRACKING')
        line.prop(properties, 'crowd', toggle=True, icon='PARTICLE_DATA')

        if properties.crowd:
            layout.template_icon_view(bpy.context.window_manager, 'crowd_type_previews', show_labels=False)
            if bpy.context.window_manager.crowd_type_previews in ['classic', 'cluster_mode']:
                layout.prop_search(properties, 'pre_pick_object', bpy.data, 'objects', text='', icon='OBJECT_DATA')
            if bpy.context.window_manager.crowd_type_previews in ['stadium']:
                layout.prop_search(properties, 'pre_pick_collection', bpy.data, 'collections', text='', icon='OUTLINER_COLLECTION')

        if properties.single:
            line = layout.row(align=True)
            line.scale_y = scale_y
            line.prop(properties, 'walk', toggle=True, icon='TRACKING_REFINE_FORWARDS')
            line.prop(properties, 'ambient', toggle=True, icon='USER')

        if not(properties.crowd and '_lite' in bpy.context.window_manager.crowd_type_previews):
            # liste personnages
            layout.prop(properties, 'style_list')

            layout.template_icon_view(bpy.context.window_manager, 'people_previews', show_labels=True)

            # liste animations
            layout.prop(properties, 'anim_list')
            layout.template_icon_view(bpy.context.window_manager, 'anim_previews', show_labels=True)

            if properties.crowd:
                layout.prop(properties, 'crowd_list')
                row = layout.row()
                row.template_list(
                    "POPULATION_UL_crowd_list",
                    "The_List",
                    properties, "crowd_list",
                    scene, "crowd_list_index"
                )
                row = layout.row(align=True)
                row.operator(POPULATION_OT_NewItem.bl_idname)
                row.operator(POPULATION_OT_RemoveItem.bl_idname)
                # row.operator(POPULATION_OT_CleanList.bl_idname)
            else:
                line = layout.row(align=True)
                line.prop(properties, 'hd', toggle=True)
                line.prop(properties, 'low', toggle=True)

            layout.prop(properties, 'randomize_color')
            line = layout.row()
            line.scale_y = scale_y
            line.operator(POPULATION_OT_import.bl_idname, icon='IMPORT')


class POPULATION_PT_Geonodes(Panel):
    "Adjustment"
    bl_idname='POPULATION_PT_Geonodes'
    bl_label = "Adjustments"
    bl_name = "Adjustments"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = 'Bproduction'
    bl_parent_id = "POPULATION_PT_Panel"

    def draw(self, context):
        mod = get_gn_mod(context.active_object)
        layout = self.layout

        if not mod:
            # box = layout.box()
            layout.label(text="No Population System selected.", icon='INFO')
            return

        col = layout.column(align=False)
        for item in mod.node_group.interface.items_tree:
            if item.in_out == 'OUTPUT' or item.identifier == "Input_0":
                continue
            # box = col.box()
            col.use_property_split = True
            col.use_property_decorate = True
            draw_gn_mod_props(col, mod, item)


def register():
    import os
    # custom icons
    global pop_custom_icons
    pop_custom_icons = bpy.utils.previews.new()
    icons_dir = os.path.join(os.path.dirname(__file__), "icons")
    pop_custom_icons.load("population", os.path.join(icons_dir, "population_icon.png"), 'IMAGE')


def unregister():
    # custom icons
    global pop_custom_icons
    bpy.utils.previews.remove(pop_custom_icons)