import bpy
from bpy.types import Operator
from bpy.types import AddonPreferences
from bpy.props import IntProperty, BoolProperty, FloatProperty, StringProperty, EnumProperty, CollectionProperty, PointerProperty
from zipfile import ZipFile
from .utilities import init_previews

def install_pack(self):
    target_filepath = bpy.context.preferences.addons[__package__].preferences.library_path

    with ZipFile(self.filepath, 'r') as zip:
        print('Extracting...')
        zip.extractall(path=target_filepath)
        print('Extracted.')

class POPULATION_AddonPreferences(AddonPreferences):
    bl_idname = __package__
    library_path: bpy.props.StringProperty(
        name="Library path",
        subtype='FILE_PATH',
    )

    def draw(self, context):
        layout = self.layout
        layout.prop(self, "library_path")
        # layout.operator(POPULATION_OT_install_pack.bl_idname)

class POPULATION_OT_install_pack(Operator):
    """Install the asset pack to the assets folder"""

    bl_idname = "population.install_pack"
    bl_label = "Install Pack"

    filepath: StringProperty(subtype="FILE_PATH", default='.zip')
    filename_ext = ".zip"
    filter_glob: StringProperty(default="*.zip", options={'HIDDEN'})

    def invoke(self, context, event):
        # check the assets dir path is set
        assets_dir = bpy.context.preferences.addons[__package__].preferences.library_path

        if assets_dir == '':
            self.report({'ERROR'}, 'Where to install assets?\nThe Assets Folder path is not set yet, please set it.')
            return {'FINISHED'}

        if not (assets_dir.endswith("\\") or assets_dir.endswith('/')):
            assets_dir += '/'
            bpy.context.preferences.addons[__package__].preferences.library_path = assets_dir

        # open browser
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}

    def execute(self, context):

        try:
            file = open(self.filepath, 'r')
            file.close()
        except:
            self.report({"ERROR"}, "Invalid file path")
            return {'FINISHED'}

        install_pack(self)
        init_previews(True, 'ALL')

        self.report({"INFO"}, "Asset pack installed!")

        return {'FINISHED'}