import bpy
import os
from bpy.props import StringProperty, BoolProperty, EnumProperty, CollectionProperty, IntProperty, FloatProperty
import bpy.utils.previews

preview_collections = {}
preview_is_loaded = False

def get_addon_preferences():
    addon_key = __package__
    if addon_key in bpy.context.preferences.addons:
        addon = bpy.context.preferences.addons[addon_key]
        if addon.preferences:
            addon_prefs = bpy.context.preferences.addons[addon_key].preferences
            return addon_prefs
        else:
            print('--- add-on does not have preferences.')
            return None
    else:
        print('--- add-on is not enabled.')
        return None

def init_previews(force = False, what = 'ALL'):
    global preview_collections
    global preview_is_loaded
    if (not preview_is_loaded or force) and get_addon_preferences():
        print('--- refresh')
        library_path = get_addon_preferences().library_path
        if what in ['CROWD_TYPE', 'ALL']:
            try:
                bpy.utils.previews.remove(preview_collections['crowd_type'])
            except:
                pass
            ### vignette des types de foule
            crowd_type_thumbnail_dir = os.path.join(library_path, 'crowd_type')
            print('--- dir', crowd_type_thumbnail_dir)
            pcoll = bpy.utils.previews.new()
            pcoll.my_previews_dir = crowd_type_thumbnail_dir
            pcoll.my_previews = ()
            preview_collections['crowd_type'] = pcoll
            preview_collections['crowd_type'] = previews_from_directory_items(preview_collections['crowd_type'])
            print('--- preview_collections', preview_collections['crowd_type'].my_previews)
            enum = EnumProperty(
                items=preview_collections['crowd_type'].my_previews,
                update=force_refresh_anim
            )
            setattr(bpy.types.WindowManager, 'crowd_type_previews', enum)
        if what in ['PERSO', 'ALL']:
            try:
                bpy.utils.previews.remove(preview_collections['people'])
            except:
                pass
            ### vignette des persos
            people_thumbnail_dir = os.path.join(library_path, 'vignette_perso')
            pcoll = bpy.utils.previews.new()
            pcoll.my_previews_dir = people_thumbnail_dir
            pcoll.my_previews = ()
            preview_collections['people'] = pcoll
            filter = [bpy.context.scene.POPULATION_Props.style_list]
            preview_collections['people'] = previews_from_directory_items(preview_collections['people'], filter)
            enum = EnumProperty(items=preview_collections['people'].my_previews)
            setattr(bpy.types.WindowManager, 'people_previews', enum)
        if what in ['ANIM', 'ALL']:
            try:
                bpy.utils.previews.remove(preview_collections['anim'])
            except:
                pass
            ### vignette des animations
            people_thumbnail_dir = os.path.join(library_path, 'vignette_animation')
            pcoll = bpy.utils.previews.new()
            pcoll.my_previews_dir = people_thumbnail_dir
            pcoll.my_previews = ()
            preview_collections['anim'] = pcoll
            filter = []
            if bpy.context.scene.POPULATION_Props.single :
                if bpy.context.scene.POPULATION_Props.ambient:
                    filter.append('ambient')
                else:
                    filter.append('oncurve')
            if bpy.context.scene.POPULATION_Props.crowd :
                if bpy.context.window_manager.crowd_type_previews == 'follow_a_curve':
                    filter.append('oncurve')
            filter.append(bpy.context.scene.POPULATION_Props.anim_list)
            preview_collections['anim'] = previews_from_directory_items(
                preview_collections['anim'],
                filter=filter)
            enum = EnumProperty(items=preview_collections['anim'].my_previews)
            setattr(bpy.types.WindowManager, 'anim_previews', enum)

        preview_is_loaded = True

def previews_from_directory_items(pcoll, filter=[]):
    enum_items = []

    if bpy.context is None:
        return enum_items

    directory = pcoll.my_previews_dir

    if directory and os.path.exists(directory):
        image_paths = []
        for fn in os.listdir(directory):
            try:
                if fn.lower().endswith(".jpg"):
                    test_filtre = True
                    if filter:
                        key_words = fn.split('#')[1]
                        for f in filter:
                            if not 'oncurve' in filter and 'oncurve' in key_words:
                                test_filtre = False
                            if f not in key_words:
                                test_filtre = False
                    if test_filtre:
                        image_paths.append(fn)
            except:pass
        for i, name in enumerate(image_paths):
            filepath = os.path.join(directory, name)
            icon = pcoll.get(name)
            if not icon:
                thumb = pcoll.load(name, filepath, 'IMAGE')
            else:
                thumb = pcoll[name]
            label = name.split('.jpg')[0].split('#')[0].replace('_', ' ')
            enum_items.append((name.split('.jpg')[0], label, "", thumb.icon_id, i))

    pcoll.my_previews = enum_items
    pcoll.my_previews_dir = directory
    return pcoll

def include_view_layers(collection, scenes="all", hide=False):

    def recur_all_layer_collection(layer_collection, all_vl, ):
        if (len(layer_collection.children) != 0):
            all_vl += layer_collection.children
            for ch in layer_collection.children:
                recur_all_layer_collection(ch, all_vl)
        return all_vl

    if (scenes == "all"):
        scenes = bpy.data.scenes[:]

    for s in scenes:
        for v in s.view_layers:
            for lc in recur_all_layer_collection(v.layer_collection, []):

                if (lc.collection == collection):

                    if (lc.exclude != hide):
                        lc.exclude = hide

def link(node_tree, from_node, from_slot_name, to_node, to_slot_name):
    input = to_node.inputs[to_slot_name]
    output = from_node.outputs[from_slot_name]
    node_tree.links.new(input, output)

def force_refresh_anim(self, context):
    init_previews(force=True, what='ANIM')